@@ -1,23 +1,77 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-
+from trytond import backend
 from trytond.model import ModelSQL, ModelView, Workflow, fields
 from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval
 from trytond.report import Report
 
-__all__ = ['Configuration', 'Package', 'Type', 'Move',
+__all__ = ['Configuration', 'ConfigurationSequence', 'Package', 'Type', 'Move',
     'ShipmentOut', 'ShipmentInReturn', 'PackageLabel']
 
 
 class Configuration:
     __metaclass__ = PoolMeta
     __name__ = 'stock.configuration'
-    package_sequence = fields.Property(fields.Many2One('ir.sequence',
-            'Package Sequence', domain=[
+    package_sequence = fields.MultiValue(fields.Many2One(
+            'ir.sequence', "Package Sequence", required=True,
+            domain=[
                 ('company', 'in', [Eval('context', {}).get('company'), None]),
                 ('code', '=', 'stock.package'),
-                ], required=True))
+                ]))
+
+    @classmethod
+    def multivalue_model(cls, field):
+        pool = Pool()
+        if field == 'package_sequence':
+            return pool.get('stock.configuration.sequence')
+        return super(Configuration, cls).multivalue_model(field)
+
+    @classmethod
+    def default_package_sequence(cls, **pattern):
+        return cls.multivalue_model(
+            'package_sequence').default_package_sequence()
+
+
+class ConfigurationSequence:
+    __metaclass__ = PoolMeta
+    __name__ = 'stock.configuration.sequence'
+    package_sequence = fields.Many2One(
+        'ir.sequence', "Package Sequence", required=True,
+        domain=[
+            ('company', 'in', [Eval('company', -1), None]),
+            ('code', '=', 'stock.package'),
+            ],
+        depends=['company'])
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+        if exist:
+            table = TableHandler(cls, module_name)
+            exist &= table.column_exist('package_sequence')
+
+        super(ConfigurationSequence, cls).__register__(module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.append('package_sequence')
+        value_names.append('package_sequence')
+        super(ConfigurationSequence, cls)._migrate_property(
+            field_names, value_names, fields)
+
+    @classmethod
+    def default_package_sequence(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id('stock_package', 'sequence_package')
+        except KeyError:
+            return None
 
 
 class Package(ModelSQL, ModelView):
@@ -115,6 +169,19 @@ class PackageMixin(object):
                 'packages': value,
                 })
 
+    @classmethod
+    def check_packages(cls, shipments):
+        for shipment in shipments:
+            if not shipment.packages:
+                continue
+            length = sum(len(p.moves) for p in shipment.packages)
+            if len(shipment.packages_moves) != length:
+                cls.raise_user_error('package_mismatch', shipment.rec_name)
+
+    @property
+    def packages_moves(self):
+        raise NotImplementedError
+
 
 class ShipmentOut(PackageMixin, object):
     __metaclass__ = PoolMeta
@@ -128,17 +195,23 @@ class ShipmentOut(PackageMixin, object):
                     'Customer Shipment "%s" are packaged.'),
                 })
 
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('packed')
+    def pack(cls, shipments):
+        super(ShipmentOut, cls).pack(shipments)
+        cls.check_packages(shipments)
+
     @classmethod
     @ModelView.button
     @Workflow.transition('done')
     def done(cls, shipments):
         super(ShipmentOut, cls).done(shipments)
-        for shipment in shipments:
-            if not shipment.packages:
-                continue
-            if (len(shipment.outgoing_moves)
-                    != sum(len(p.moves) for p in shipment.packages)):
-                cls.raise_user_error('package_mismatch', shipment.rec_name)
+        cls.check_packages(shipments)
+
+    @property
+    def packages_moves(self):
+        return self.outgoing_moves
 
 
 class ShipmentInReturn(PackageMixin, object):
@@ -149,7 +222,7 @@ class ShipmentInReturn(PackageMixin, object):
     def __setup__(cls):
         super(ShipmentInReturn, cls).__setup__()
         cls._error_messages.update({
-                'package_mismatch': ('Not all Outgoing Moves of '
+                'package_mismatch': ('Not all Moves of '
                     'Supplier Return Shipment "%s" are packaged.'),
                 })
 
@@ -158,12 +231,11 @@ class ShipmentInReturn(PackageMixin, object):
     @Workflow.transition('done')
     def done(cls, shipments):
         super(ShipmentInReturn, cls).done(shipments)
-        for shipment in shipments:
-            if not shipment.packages:
-                continue
-            if (len(shipment.moves)
-                    != sum(len(p.moves) for p in shipment.packages)):
-                cls.raise_user_error('package_mismatch', shipment.rec_name)
+        cls.check_packages(shipments)
+
+    @property
+    def packages_moves(self):
+        return self.moves
 
 
 class PackageLabel(Report):
